﻿Type=Class
Version=5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Dim Actor As sgeActor
	Dim OpenMouth As sgeRegion
	Dim Lives As Byte
	Dim HasToEat As Int
	Dim CurrentDirection, NextDirection As Byte
	Dim Speed As Double
	Private AnimHorizontal, AnimVertical As sgeAnimation
	Private Const FRAME_DURATION As Float = 300
End Sub

'Initializes the droid instance
Public Sub Initialize
	'Splits the sprite sheet into regions
	Dim Droids() As sgeRegion= Main.AM.Split("Droid", 4, 1)
	OpenMouth = Droids(1) 'static image for lives

	'Creates the animations with these regions
	AnimHorizontal.Initialize2(Array As sgeRegion(Droids(0), Droids(1)), FRAME_DURATION, AnimHorizontal.REPEAT_INFINITE)
	AnimVertical.Initialize2(Array As sgeRegion(Droids(2), Droids(3)), FRAME_DURATION, AnimVertical.REPEAT_INFINITE)

	'Creates the animated actor
	Actor.InitializeWithAnimation(AnimHorizontal, Main.SGE.Root, "PacDroid")

	'The actor is hidden when the game begins
	Actor.Visible = False
End Sub

'Resets the droid data
Public Sub Reset
	Actor.SetAnimation(AnimHorizontal)
	Actor.FlipX = False
	Actor.FlipY = False
	Actor.DrawingOffsetX = 5
	Actor.DrawingOffsetY = 5
	Actor.SetScale(1)
	CurrentDirection = Main.Direction_NONE
	NextDirection = Main.Direction_NONE
End Sub

'Moves the droid
Public Sub Move(Elapsed As Double)
	Dim WantTakeBend As Boolean = False
	If CurrentDirection = Main.Direction_NONE Then
		'It's time for the droid to start moving
		CurrentDirection = NextDirection
		NextDirection = Main.Direction_NONE
	Else If CurrentDirection <> NextDirection And NextDirection <> Main.Direction_NONE Then
		'The player wants to move in another direction
		If (CurrentDirection = Main.Direction_RIGHT And NextDirection = Main.Direction_LEFT) _
		Or (CurrentDirection = Main.Direction_LEFT And NextDirection = Main.Direction_RIGHT) _
		Or (CurrentDirection = Main.Direction_TOP And NextDirection = Main.Direction_BOTTOM) _
		Or (CurrentDirection = Main.Direction_BOTTOM And NextDirection = Main.Direction_TOP) Then
			'It's the opposite direction
			CurrentDirection = NextDirection
			NextDirection = Main.Direction_NONE
		Else
			'The player wants to take a bend
			If NextDirection = Main.Direction_TOP Then
				WantTakeBend = CanMoveTo(Actor.X, Actor.Y - Main.TILESIZE)
			Else If NextDirection = Main.Direction_BOTTOM Then
				WantTakeBend = CanMoveTo(Actor.X, Actor.Y + Main.TILESIZE)
			Else If NextDirection = Main.Direction_LEFT Then
				WantTakeBend = CanMoveTo(Actor.X - Main.TILESIZE, Actor.Y)
			Else If NextDirection = Main.Direction_RIGHT Then
				WantTakeBend = CanMoveTo(Actor.X + Main.TILESIZE, Actor.Y)
			End If
		End If
	End If
	Dim Limit As Int
	If CurrentDirection = Main.Direction_RIGHT Then
		If WantTakeBend Then
			'Finishes the move on the current tile before changing direction
			Limit = NextMoveX(0)
		Else
			'Continues moving
			Limit = NextMoveX(Main.TILESIZE)
		End If
		If Actor.X = Limit Then
			CurrentDirection = Main.Direction_NONE
		Else
			If Actor.VisualObject <> AnimHorizontal Then
				Actor.SetAnimation(AnimHorizontal)
				Actor.FlipY = False
				Actor.DrawingOffsetY = 5
			End If
			Actor.FlipX = False
			Actor.DrawingOffsetX = 5
			Actor.X = Min(Actor.X + Speed * Elapsed, Limit)
		End If
	Else If CurrentDirection = Main.Direction_LEFT Then
		If WantTakeBend Then
			'Finishes the move on the current tile before changing direction
			Limit = NextMoveX(0)
		Else
			'Continues moving
			Limit = NextMoveX(-Main.TILESIZE)
		End If
		If Actor.X = Limit Then
			CurrentDirection = Main.Direction_NONE
		Else
			If Actor.VisualObject <> AnimHorizontal Then
				Actor.SetAnimation(AnimHorizontal)
				Actor.FlipY = False
				Actor.DrawingOffsetY = 5
			End If
			Actor.FlipX = True
			Actor.DrawingOffsetX = -5
			Actor.X = Max(Actor.X - Speed * Elapsed, Limit)
		End If
	Else If CurrentDirection = Main.Direction_TOP Then
		If WantTakeBend Then
			'Finishes the move on the current tile before changing direction
			Limit = NextMoveY(0)
		Else
			'Continues moving
			Limit = NextMoveY(-Main.TILESIZE)
		End If
		If Actor.Y = Limit Then
			CurrentDirection = Main.Direction_NONE
		Else
			If Actor.VisualObject <> AnimVertical Then
				Actor.SetAnimation(AnimVertical)
				Actor.FlipX = False
				Actor.DrawingOffsetX = 5
			End If
			Actor.FlipY = True
			Actor.DrawingOffsetY = -5
			Actor.Y = Max(Actor.Y - Speed * Elapsed, Limit)
		End If
	Else If CurrentDirection = Main.Direction_BOTTOM Then
		If WantTakeBend Then
			'Finishes the move on the current tile before changing direction
			Limit = NextMoveY(0)
		Else
			'Continues moving
			Limit = NextMoveY(Main.TILESIZE)
		End If
		If Actor.Y = Limit Then
			CurrentDirection = Main.Direction_NONE
		Else
			If Actor.VisualObject <> AnimVertical Then
				Actor.SetAnimation(AnimVertical)
				Actor.FlipX = False
				Actor.DrawingOffsetX = 5
			End If
			Actor.FlipY = False
			Actor.DrawingOffsetY = 5
			Actor.Y = Min(Actor.Y + Speed * Elapsed, Limit)
		End If
	End If
End Sub

Private Sub CanMoveTo(X As Int, Y As Int) As Boolean
	'Checks whether the droid can move to the given tile
	Dim TileX As Int = Main.Maze.FromPixelToTile_X(X)
	Dim TileY As Int = Main.Maze.FromPixelToTile_Y(Y)
	Dim Tile As sgeMapTile = Main.Maze.GetTile(TileX, TileY)
	Return Main.Maze.CanMoveTo(Tile)
End Sub

Private Sub NextMoveX(DeltaX As Int) As Int
	'Checks whether the droid can move to the next tile
	If CanMoveTo(Actor.X + DeltaX, Actor.Y) Then
		'Returns the next tile position
		Return Main.Maze.FromTileToPixel_X(Main.Maze.FromPixelToTile_X(Actor.X + DeltaX))
	Else
		'Returns the current tile position
		Return Main.Maze.FromTileToPixel_X(Main.Maze.FromPixelToTile_X(Actor.X))
	End If
End Sub

Private Sub NextMoveY(DeltaY As Int) As Int
	'Checks whether the droid can move to the next tile
	If CanMoveTo(Actor.X, Actor.Y + DeltaY) Then
		'Returns the next tile position
		Return Main.Maze.FromTileToPixel_Y(Main.Maze.FromPixelToTile_Y(Actor.Y + DeltaY))
	Else
		'Returns the current tile position
		Return Main.Maze.FromTileToPixel_Y(Main.Maze.FromPixelToTile_Y(Actor.Y))
	End If
End Sub

Sub Droid_Caught
	'If still alive...
	If Lives >= 0 Then
		'It gets another chance
		Main.Another_Chance
	End If
End Sub
