﻿Type=Class
Version=5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Dim Actor As sgeActor
	Dim MyAppearance As sgeRegion
	Dim MyIndex As Int
	Dim Mode, PreviousMode As Byte
	Dim ModeTime, PreviousModeTime As Float
	Dim ScatterCount As Byte
	Private ScatterTargetX, ScatterTargetY As Int
	Private TargetX, TargetY As Int
	Private MovingDirection As Byte
	Private CurrentTile As sgeMapTile = Null
	Type typMove(Possible As Boolean, Coordinates As sgePoint2D, Distance As Float)
End Sub

'Initializes the ghost instance
Public Sub Initialize(Regions() As sgeRegion, Index As Int)
	MyIndex = Index

	'Creates the actor
	MyAppearance = Regions(MyIndex)
	Actor.InitializeWithRegion(MyAppearance, Main.SGE.Root, "Ghost")
	Actor.DrawingOffsetX = 5
	Actor.DrawingOffsetY = 5

	'The actor is hidden when the game begins
	Actor.Visible = False

	'Sets the coordinates of the ghost target in Scatter mode
	Dim Map As sgeTiledMap = Main.AM.GetTiledMap("Maps")
	If Index = 0 Then
		SetScatterTarget(1, 1)
	Else if Index = 1 Then
		SetScatterTarget(1, Map.MapHeight - 2)
	Else if Index = 2 Then
		SetScatterTarget(Map.MapWidth - 2, 1)
	Else if Index = 3 Then
		SetScatterTarget(Map.MapWidth - 2, Map.MapHeight - 2)
	End If	
End Sub

'Sets the coordinates (in tiles) of the target in Scatter mode
Private Sub SetScatterTarget(TileX As Int, TileY As Int)
	ScatterTargetX = Main.Maze.FromTileToPixel_X(TileX)
	ScatterTargetY = Main.Maze.FromTileToPixel_Y(TileY)
End Sub

'Resets the ghost mode
Public Sub ResetMode
	ChangeMode(Main.Mode_SCATTER)
	ModeTime = 0
	ScatterCount = 3
End Sub

'Resets the ghost target
Public Sub ResetTarget
	MovingDirection = Main.Direction_NONE
	TargetX = -1
	TargetY = -1
End Sub

'Changes the behavior and appearance of the ghost
Public Sub ChangeMode(NewMode As Byte)
	If Mode <> NewMode Then
		If Actor.Action <> Null Then
			'Stops the action and forces the visibility (the visible state is unknown after a blinking in Frightened mode)
			Actor.ClearActions
			Actor.Visible = True
		End If
		If NewMode = Main.Mode_FRIGHTENED Then
			'The ghost turns to blue
			PreviousMode = Mode
			PreviousModeTime = ModeTime
			Actor.SetRegion(Main.ScaredGhost)
		Else If NewMode = Main.Mode_RETURN Then
			'The ghost becomes translucent
			Actor.Alpha = 0.4
		Else If Mode = Main.Mode_FRIGHTENED Or Mode = Main.Mode_RETURN Then
			'Back to the normal appearance
			Actor.SetRegion(MyAppearance)
			Actor.Alpha = 1

			'Back to the previous mode
			Mode = PreviousMode
			ModeTime = PreviousModeTime
			Return
		End If
		Mode = NewMode
		ModeTime = 0
	Else If Mode = Main.Mode_FRIGHTENED Then
		'Resets the time counter when the ghost is already frightened
		Actor.ClearActions
		Actor.Visible = True
		ModeTime = 0
	End If
End Sub

Private Sub IsNotOccupied(TileX As Int, TileY As Int) As Boolean
	'Checks whether the given tile is not occupied by another ghost
	If Mode = Main.Mode_RETURN Then
		Return True 'Does not matter in Return mode
	End If
	For i = 0 To Main.Ghosts.Length - 1
		If i <> MyIndex And Main.Ghosts(i).Mode <> Main.Mode_RETURN Then
			Dim tX, tY As Int
			If Main.Ghosts(i).TargetX = -1 Or Main.Ghosts(i).TargetY = -1 Then
				tX = Main.Maze.FromPixelToTile_X(Main.Ghosts(i).Actor.X)
				tY = Main.Maze.FromPixelToTile_Y(Main.Ghosts(i).Actor.Y)
			Else
				tX = Main.Maze.FromPixelToTile_X(Main.Ghosts(i).TargetX)
				tY = Main.Maze.FromPixelToTile_Y(Main.Ghosts(i).TargetY)
			End If
			If tX = TileX And tY = TileY Then
				'Already occupied
				Return False
			End If
		End If
	Next
	Return True
End Sub

Private Sub CanMoveTo(Tile As sgeMapTile, IsInTheHouse As Boolean) As Boolean
	'Checks whether the given tile is not a wall
	If Tile = Null Then Return False 'Teleportation tile
	Return Main.Maze.CanMoveTo(Tile) Or ((IsInTheHouse Or Mode = Main.Mode_RETURN) _
	                                 And (Tile = Main.Maze.RefDoorTile Or Tile = Main.Maze.RefHouseTile))
End Sub

Private Sub FindPossibleMoves As typMove(4)
	'Creates an array of possible moves with their future coordinates
	Dim Moves(4) As typMove
	Dim TileX As Int = Main.Maze.FromPixelToTile_X(Actor.X)
	Dim TileY As Int = Main.Maze.FromPixelToTile_Y(Actor.Y)
	CurrentTile = Main.Maze.GetTile(TileX, TileY)
	Dim IsInTheHouse As Boolean = (CurrentTile = Main.Maze.RefHouseTile)
	Moves(Main.Direction_RIGHT).Initialize
	If MovingDirection <> Main.Direction_LEFT Then
		'Checks the tile on the right
		Dim Tile As sgeMapTile = Main.Maze.GetTile(TileX + 1, TileY)
		If CanMoveTo(Tile, IsInTheHouse) And IsNotOccupied(TileX + 1, TileY) Then
			Moves(Main.Direction_RIGHT).Possible = True
			Moves(Main.Direction_RIGHT).Coordinates.Initialize(TileX + 1, TileY)
		End If
	End If
	Moves(Main.Direction_LEFT).Initialize
	If MovingDirection <> Main.Direction_RIGHT Then
		'Checks the tile on the left
		Dim Tile As sgeMapTile = Main.Maze.GetTile(TileX - 1, TileY)
		If CanMoveTo(Tile, IsInTheHouse) And IsNotOccupied(TileX - 1, TileY) Then
			Moves(Main.Direction_LEFT).Possible = True
			Moves(Main.Direction_LEFT).Coordinates.Initialize(TileX - 1, TileY)
		End If
	End If
	Moves(Main.Direction_TOP).Initialize
	If MovingDirection <> Main.Direction_BOTTOM Then
		'Checks the tile above
		Dim Tile As sgeMapTile = Main.Maze.GetTile(TileX, TileY - 1)
		If CanMoveTo(Tile, IsInTheHouse) And IsNotOccupied(TileX, TileY - 1) Then
			Moves(Main.Direction_TOP).Possible = True
			Moves(Main.Direction_TOP).Coordinates.Initialize(TileX, TileY - 1)
		End If
	End If
	Moves(Main.Direction_BOTTOM).Initialize
	If MovingDirection <> Main.Direction_TOP Then
		'Checks the tile below
		Dim Tile As sgeMapTile = Main.Maze.GetTile(TileX, TileY + 1)
		If CanMoveTo(Tile, IsInTheHouse) And IsNotOccupied(TileX, TileY + 1) Then
			Moves(Main.Direction_BOTTOM).Possible = True
			Moves(Main.Direction_BOTTOM).Coordinates.Initialize(TileX, TileY + 1)
		End If
	End If
	Return Moves
End Sub

Private Sub Move(Elapsed As Double)
	'Moves to the targeted tile
	If MovingDirection = Main.Direction_LEFT Then
		Actor.X = Max(Actor.X - Main.Ghost_Speed * Elapsed, TargetX)
	Else If MovingDirection = Main.Direction_RIGHT Then
		Actor.X = Min(Actor.X + Main.Ghost_Speed * Elapsed, TargetX)
	Else If MovingDirection = Main.Direction_BOTTOM Then
		Actor.Y = Min(Actor.Y + Main.Ghost_Speed * Elapsed, TargetY)
	Else If MovingDirection = Main.Direction_TOP Then
		Actor.Y = Max(Actor.Y - Main.Ghost_Speed * Elapsed, TargetY)
	End If
End Sub

Private Sub ContinueMove(Elapsed As Double) As Boolean
	If TargetX = -1 Or TargetY = -1 Or (Actor.X = TargetX And Actor.Y = TargetY) Then
		'The targeted tile has been reached so the ghost can select a new tile
		Return False
	Else
		'Continues the move to the targeted tile
		Move(Elapsed)
		Return True
	End If
End Sub

Private Sub Teleport(Elapsed As Double)
	'The ghost is moved to the other side of the tunnel
	If CurrentTile = Main.Maze.RefTeleportTile Then
		'Horizontal teleporter
		Dim TileX As Int = Main.Maze.FromPixelToTile_X(Actor.X)
		If MovingDirection = Main.Direction_LEFT Then
			TileX = TileX + 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX - 1)
		Else
			TileX = TileX - 26
			TargetX = Main.Maze.FromTileToPixel_X(TileX + 1)
		End If
		Actor.X = Main.Maze.FromTileToPixel_X(TileX)
	Else
		'Vertical teleporter
		Dim TileY As Int = Main.Maze.FromPixelToTile_Y(Actor.Y)
		If MovingDirection = Main.Direction_TOP Then
			TileY = TileY - 24
			TargetY = Main.Maze.FromTileToPixel_Y(TileY - 1)
			Actor.X = Actor.X - Main.TILESIZE
		Else
			TileY = TileY + 24
			TargetY = Main.Maze.FromTileToPixel_Y(TileY + 1)
			Actor.X = Actor.X + Main.TILESIZE
		End If
		TargetX = Actor.X
		Actor.Y = Main.Maze.FromTileToPixel_Y(TileY)
	End If
	Move(Elapsed)
End Sub

Private Sub MoveCloserTo(Position As sgePoint2D, Elapsed As Double)
	'Is the current move finished?
	If ContinueMove(Elapsed) Then Return

	'Creates an array of possible moves
	Dim Moves(4) As typMove = FindPossibleMoves

	'If the ghost is in the house, the outside area near the door replaces the given position
	If CurrentTile = Main.Maze.RefHouseTile Then
		Dim TileDoorX As Int = Main.Maze.CurrentLayer.Properties.Get("doorX")
		Dim TileDoorY As Int = Main.Maze.CurrentLayer.Properties.Get("doorY")
		If Main.Maze.FromPixelToTile_X(Actor.X) > TileDoorX Then
			TileDoorX = TileDoorX + 1
		End If
		Position.Initialize(Main.Maze.FromTileToPixel_X(TileDoorX), _
		                    Main.Maze.FromTileToPixel_Y(TileDoorY - 1))
	'If the ghost uses a teleporter...
	Else If CurrentTile = Main.Maze.RefTeleportTile Or CurrentTile = Main.Maze.RefTeleport2Tile Then
		Teleport(Elapsed)
		Return
	End If

	'Selects the shortest path between the ghost and the given position
	Dim Shortest_Idx As Int = -1
	For i = 0 To 3
		If Moves(i).Possible Then
			Moves(i).Coordinates.Initialize(Main.Maze.FromTileToPixel_X(Moves(i).Coordinates.X), _
			                                Main.Maze.FromTileToPixel_Y(Moves(i).Coordinates.Y))
			Moves(i).Distance = Position.Distance2(Moves(i).Coordinates)
			If Shortest_Idx = -1 Or Moves(i).Distance < Moves(Shortest_Idx).Distance Then
				Shortest_Idx = i
			End If
		End If
	Next

	'Moves to the selected tile
	If Shortest_Idx > -1 Then
		MovingDirection = Shortest_Idx
		TargetX = Moves(Shortest_Idx).Coordinates.X
		TargetY = Moves(Shortest_Idx).Coordinates.Y
		Move(Elapsed)
	Else
		'The ghost is blocked by another ghost so it is allowed to change direction
		MovingDirection = Main.Direction_NONE
	End If
End Sub

Public Sub MoveInChaseMode(Elapsed As Double)
	Dim PosDroid As sgePoint2D
	PosDroid.Initialize(Main.PacDroid.Actor.X, Main.PacDroid.Actor.Y)
	MoveCloserTo(PosDroid, Elapsed)
End Sub

Public Sub MoveInScatterMode(Elapsed As Double)
	Dim PosTarget As sgePoint2D
	PosTarget.Initialize(ScatterTargetX, ScatterTargetY)
	MoveCloserTo(PosTarget, Elapsed)
End Sub

Public Sub MoveInFrightenedMode(Elapsed As Double)
	'Is the current move finished?
	If ContinueMove(Elapsed) Then Return

	'Creates an array of possible moves
	Dim Moves(4) As typMove = FindPossibleMoves

	'If the ghost uses a teleporter...
	If CurrentTile = Main.Maze.RefTeleportTile Or CurrentTile = Main.Maze.RefTeleport2Tile Then
		Teleport(Elapsed)
		Return
	End If

	'Selects the path that leads the ghost away from the droid
	Dim PosDroid As sgePoint2D
	PosDroid.Initialize(Main.PacDroid.Actor.X, Main.PacDroid.Actor.Y)
	Dim Farthest_Idx As Int = -1
	For i = 0 To 3
		If Moves(i).Possible Then
			Moves(i).Coordinates.Initialize(Main.Maze.FromTileToPixel_X(Moves(i).Coordinates.X), _
			                                Main.Maze.FromTileToPixel_Y(Moves(i).Coordinates.Y))
			Moves(i).Distance = PosDroid.Distance2(Moves(i).Coordinates)
			If Farthest_Idx = -1 Or Moves(i).Distance > Moves(Farthest_Idx).Distance Then
				Farthest_Idx = i
			End If
		End If
	Next

	'Moves to the selected tile
	If Farthest_Idx > -1 Then
		MovingDirection = Farthest_Idx
		TargetX = Moves(Farthest_Idx).Coordinates.X
		TargetY = Moves(Farthest_Idx).Coordinates.Y
		Move(Elapsed)
	Else
		'The ghost is blocked by another ghost so it is allowed to change direction
		MovingDirection = Main.Direction_NONE
	End If
End Sub

Public Sub MoveInReturnMode(Elapsed As Double)
	Dim TileDoorX As Int = Main.Maze.CurrentLayer.GetProperty("doorX")
	Dim TileDoorY As Int = Main.Maze.CurrentLayer.GetProperty("doorY")
	If Main.Maze.FromPixelToTile_X(Actor.X) > TileDoorX Then
		TileDoorX = TileDoorX + 1
	End If
	Dim PosHouse As sgePoint2D
	PosHouse.Initialize(Main.Maze.FromTileToPixel_X(TileDoorX), Main.Maze.FromTileToPixel_Y(TileDoorY + 1))
	MoveCloserTo(PosHouse, Elapsed)
End Sub
