﻿Type=Class
Version=5
ModulesStructureVersion=1
B4J=true
@EndOfDesignText@
Sub Class_Globals
	Private fx As JFX
	Private Maps As sgeTiledMap
	Dim CurrentLayer As sgeTiledMapTileLayer = Null
	Dim RefFoodTile As sgeMapTile = Null
	Dim RefEnergizerTile As sgeMapTile = Null
	Dim RefEmptyTile As sgeMapTile = Null
	Dim RefTeleportTile As sgeMapTile = Null
	Dim RefTeleport2Tile As sgeMapTile = Null
	Dim RefHouseTile As sgeMapTile = Null
	Dim RefDoorTile As sgeMapTile = Null
	Private TileMap As Map
End Sub

'Initializes the maze instance
Public Sub Initialize(TiledMaps As sgeTiledMap)
	Maps = TiledMaps

	'Initializes the Map storing the tiles of each level
	TileMap.Initialize

	'Selects the first maze
	SelectLayer(0)

	'Gets the reference tiles
	RefEmptyTile = Maps.GetTile(29)
	RefFoodTile = Maps.GetTile(33)
	RefDoorTile = Maps.GetTile(283)
	RefHouseTile = Maps.GetTile(308)
	RefTeleportTile = Maps.GetTile(399)
	RefTeleport2Tile = Maps.GetTile(372)
	RefEnergizerTile = Maps.GetTile(426)
End Sub

'Selects a maze. Each layer is a different maze.
Public Sub SelectLayer(Layer As Int)
	Layer = Min(Layer, Maps.Layers.Size)
	CurrentLayer = Maps.Layers.Get(Layer)
	If TileMap.ContainsKey(Layer) Then
		'All cells are restored to their initial tile
		Dim Tiles(,) As Object = TileMap.Get(Layer)
		For x = 0 To Maps.MapWidth - 1
			For y = 0 To Maps.MapHeight - 1
				Dim Cell As sgeMapCell = CurrentLayer.GetCell(x, y)
				If Cell <> Null Then Cell.Tile = Tiles(x, y)
			Next
		Next
	Else
		'All cell tiles are saved in an array
		Dim Tiles(Maps.MapWidth, Maps.MapHeight) As Object
		For x = 0 To Maps.MapWidth - 1
			For y = 0 To Maps.MapHeight - 1
				Tiles(x, y) = GetTile(x, y)
			Next
		Next
		TileMap.Put(Layer, Tiles)
	End If
End Sub

'Counts the number of things to eat in the current level
Public Sub CountFood As Int
	Dim Food As Int = 0
	For x = 0 To Maps.MapWidth - 1
		For y = 0 To Maps.MapHeight - 1
			Dim Tile As sgeMapTile = GetTile(x, y)
			If Tile <> Null And (Tile = RefFoodTile Or Tile = RefEnergizerTile) Then
				Food = Food + 1
			End If
		Next
	Next
	Return Food
End Sub

Public Sub FromTileToPixel_X(TileX As Int) As Int
	Return Round((TileX - 0.5) * Main.TILESIZE)
End Sub

Public Sub FromTileToPixel_Y(TileY As Int) As Int
	Return Round((TileY - 0.5) * Main.TILESIZE) + Main.SCORE_HEIGHT
End Sub

Public Sub FromPixelToTile_X(PixelX As Int) As Int
	Return Round((PixelX / Main.TILESIZE) + 0.5)
End Sub

Public Sub FromPixelToTile_Y(PixelY As Int) As Int
	Return Round(((PixelY - Main.SCORE_HEIGHT) / Main.TILESIZE) + 0.5)
End Sub

'Checks whether the given tile allows to move (it's not a wall)
Public Sub CanMoveTo(Tile As sgeMapTile) As Boolean
	Return (Tile = RefFoodTile Or Tile = RefEnergizerTile Or _
	        Tile = RefEmptyTile Or Tile = RefTeleportTile Or Tile = RefTeleport2Tile)
End Sub

'Gets the tile at the given coordinates
Public Sub GetTile(TileX As Int, TileY As Int) As sgeMapTile
	Try
		Return CurrentLayer.GetCell(TileX, TileY).Tile
	Catch
		Return Null
	End Try
End Sub

'Draws the current layer
Public Sub Draw(G As sgeGraphics)
	G.DrawTiledMapLayer2(CurrentLayer, 0, Main.SCORE_HEIGHT)
End Sub
